@@ -156,6 +156,19 @@ get_mountable_info_for_type (GDaemonVfs *vfs,
   return NULL;
 }
 
+static void
+str_tolower_inplace (char *str)
+{
+  char *p = str;
+
+  while (*p != 0)
+    {
+      *p = g_ascii_tolower (*p);
+      p++;
+    }
+
+}
+
 static gboolean
 get_mountspec_from_uri (GDaemonVfs *vfs,
 			const char *uri,
@@ -171,6 +184,10 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
   scheme = g_uri_parse_scheme (uri);
   if (scheme == NULL)
     return FALSE;
+
+  /* convert the scheme to lower case since g_uri_parse_scheme
+   * doesn't do that and we compare with g_str_equal */
+  str_tolower_inplace (scheme);
   
   spec = NULL;
   path = NULL;
@@ -193,7 +210,7 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
     {
       GDecodedUri *decoded;
       MountableInfo *mountable;
-      char *type, *p;
+      char *type;
       int l;
 
       decoded = g_vfs_decode_uri (uri);
@@ -213,8 +230,7 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
 	      if (mountable && mountable->host_is_inet)
 		{
 		  /* Convert hostname to lower case */
-		  for (p = decoded->host; *p != 0; p++)
-		    *p = g_ascii_tolower (*p);
+          str_tolower_inplace (decoded->host);
 		  
 		  /* Remove brackets aroung ipv6 addresses */
 		  l = strlen (decoded->host);
@@ -240,6 +256,11 @@ get_mountspec_from_uri (GDaemonVfs *vfs,
 	      g_mount_spec_set (spec, "port", port);
 	      g_free (port);
 	    }
+
+	  if (decoded->query && *decoded->query)
+	    g_mount_spec_set (spec, "query", decoded->query);
+	  if (decoded->fragment && *decoded->fragment)
+	    g_mount_spec_set (spec, "fragment", decoded->fragment);
 	  
 	  path = g_strdup (decoded->path);
 	  
@@ -356,7 +377,7 @@ g_daemon_vfs_get_file_for_uri (GVfs       *vfs,
 
   daemon_vfs = G_DAEMON_VFS (vfs);
 
-  if (g_str_has_prefix (uri, "file:"))
+  if (g_ascii_strncasecmp (uri, "file:", 5) == 0)
     {
       path = g_filename_from_uri (uri, NULL, NULL);
 
@@ -483,6 +504,9 @@ _g_daemon_vfs_get_uri_for_mountspec (GMountSpec *spec,
 	decoded.path = "/";
       else
 	decoded.path = path;
+
+      decoded.query = (char *)g_mount_spec_get (spec, "query");
+      decoded.fragment = (char *)g_mount_spec_get (spec, "fragment");
       
       uri = g_vfs_encode_uri (&decoded, FALSE);
       
@@ -952,11 +976,11 @@ void g_vfs_uri_mapper_sftp_register (GIOModule *module);
 void
 g_io_module_load (GIOModule *module)
 {
- /* This is so that system daemons can use gio
-  * without spawning private dbus instances.
-  * See bug 526454.
-  */
-  if (g_getenv ("DBUS_SESSION_BUS_ADDRESS") == NULL)
+  /* This is so that system daemons can use gio
+   * without spawning private dbus instances.
+   * See bug 526454.
+   */
+  if (g_getenv ("DBUS_SESSION_BUS_ADDRESS") == NULL) 
     return;
 
   g_daemon_vfs_register_type (G_TYPE_MODULE (module));
