@@ -1,5 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-
 /* gvfsfusedaemon.c - FUSE file system mapping daemon for GVFS
  * 
  * Copyright (C) 2007-2008 Hans Petter Jansson
@@ -313,9 +311,11 @@ reindex_file_handle_for_path (const gchar *old_path, const gchar *new_path)
                                      (gpointer *) &fh))
       goto out;
 
+  g_hash_table_steal (global_fh_table, old_path);
+
   g_free (fh->path);
   fh->path = g_strdup (new_path);
-  g_hash_table_steal (global_fh_table, old_path);
+
   g_hash_table_insert (global_fh_table, fh->path, fh);
 
  out:
@@ -666,9 +666,6 @@ vfs_statfs (const gchar *path, struct statvfs *stbuf)
   stbuf->f_flag = 0;  /* Ignored by FUSE */
   stbuf->f_namemax = 1024;
 
-  if (statvfs ("/", stbuf) != 0)
-    result = -errno;
-
   debug_print ("vfs_statfs: -> %s\n", strerror (-result));
 
   return result;
@@ -959,11 +956,15 @@ vfs_open (const gchar *path, struct fuse_file_info *fi)
 
               /* Set up a stream here, so we can check for errors */
 
+              g_mutex_lock (fh->mutex);
+
               if (fi->flags & O_WRONLY || fi->flags & O_RDWR)
                 result = setup_output_stream (file, fh);
               else
                 result = setup_input_stream (file, fh);
 
+              g_mutex_unlock (fh->mutex);
+
               /* The added reference to the file handle is released in vfs_release() */
             }
           else if (file_type == G_FILE_TYPE_DIRECTORY)
@@ -1047,11 +1048,14 @@ vfs_create (const gchar *path, mode_t mode, struct fuse_file_info *fi)
 
               SET_FILE_HANDLE (fi, fh);
 
-              g_assert (fh->stream == NULL);
+              g_mutex_lock (fh->mutex);
 
+              file_handle_close_stream (fh);
               fh->stream = file_output_stream;
               fh->op = FILE_OP_WRITE;
 
+              g_mutex_unlock (fh->mutex);
+
               /* The added reference to the file handle is released in vfs_release() */
             }
           else
@@ -2033,6 +2037,7 @@ subthread_main (gpointer data)
   g_signal_handlers_disconnect_by_func (volume_monitor, mount_tracker_unmounted_cb, NULL);
 
   g_main_loop_unref (subthread_main_loop);
+  subthread_main_loop = NULL;
 
   g_object_unref (volume_monitor);
   volume_monitor = NULL;
@@ -2144,7 +2149,8 @@ static void
 vfs_destroy (gpointer param)
 {
   mount_list_free ();
-  g_main_loop_quit (subthread_main_loop);
+  if (subthread_main_loop != NULL) 
+    g_main_loop_quit (subthread_main_loop);
   g_mutex_free (mount_list_mutex);
   g_object_unref (gvfs);
 }
