@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* GIO - GLib Input, Output and Streaming Library
  * 
  * Copyright (C) 2008 Red Hat, Inc.
@@ -231,8 +230,8 @@ g_vfs_job_failed_from_http_status (GVfsJob *job, guint status_code, const char *
   switch (status_code) {
 
   case SOUP_STATUS_NOT_FOUND:
-    g_vfs_job_failed (job, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                      message);
+    g_vfs_job_failed_literal (job, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+                              message);
     break;
 
   case SOUP_STATUS_UNAUTHORIZED:
@@ -341,10 +340,10 @@ open_for_read_ready (GObject      *source_object,
                                        &error);
   if (res == FALSE)
     {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        error->domain,
-                        error->code,
-                        error->message);
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                error->domain,
+                                error->code,
+                                error->message);
 
       g_error_free (error);
       return;
@@ -402,10 +401,10 @@ read_ready (GObject      *source_object,
 
   if (nread < 0)
    {
-     g_vfs_job_failed (G_VFS_JOB (job),
-                       error->domain,
-                       error->code,
-                       error->message);
+     g_vfs_job_failed_literal (G_VFS_JOB (job),
+                               error->domain,
+                               error->code,
+                               error->message);
 
      g_error_free (error);
      return;
@@ -452,10 +451,10 @@ try_seek_on_read (GVfsBackend *backend,
   if (!g_seekable_seek (G_SEEKABLE (stream), offset, type,
                         G_VFS_JOB (job)->cancellable, &error))
     {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        error->domain,
-                        error->code,
-                        error->message);
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                error->domain,
+                                error->code,
+                                error->message);
       g_error_free (error);
       return FALSE;
     }
@@ -486,10 +485,10 @@ close_read_ready (GObject      *source_object,
                                      &error);
   if (res == FALSE)
     {
-      g_vfs_job_failed (G_VFS_JOB (job),
-                        error->domain,
-                        error->code,
-                        error->message);
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                error->domain,
+                                error->code,
+                                error->message);
 
       g_error_free (error);
     }
@@ -608,9 +607,16 @@ query_info_ready (SoupSession *session,
                                    "Last-Modified");
   if (text)
     {
+      SoupDate *sd;
       GTimeVal tv;
-      if (g_time_val_from_iso8601 (text, &tv))
-        g_file_info_set_modification_time (info, &tv);
+
+      sd = soup_date_new_from_string(text);
+      if (sd)
+        {
+          soup_date_to_timeval (sd, &tv);
+	  g_file_info_set_modification_time (info, &tv);
+          soup_date_free (sd);
+        }
     }
 
 
